@@ -17,7 +17,6 @@
    USA.
 */
 
-#define _GNU_SOURCE
 #include "defs.h"
 #include <ctype.h>
 #include <pwd.h>
@@ -68,6 +67,8 @@ static boolean parse_cnewer PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_comma PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_ctime PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_daystart PARAMS((char *argv[], int *arg_ptr));
+static boolean parse_delete PARAMS((char *argv[], int *arg_ptr));
+static boolean parse_d PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_depth PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_empty PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_exec PARAMS((char *argv[], int *arg_ptr));
@@ -123,6 +124,7 @@ static boolean parse_xdev PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_ignore_race PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_noignore_race PARAMS((char *argv[], int *arg_ptr));
 static boolean parse_xtype PARAMS((char *argv[], int *arg_ptr));
+static boolean parse_quit PARAMS((char *argv[], int *arg_ptr));
 
 static boolean insert_regex PARAMS((char *argv[], int *arg_ptr, boolean ignore_case));
 static boolean insert_type PARAMS((char *argv[], int *arg_ptr, boolean (*which_pred )()));
@@ -170,6 +172,8 @@ static struct parser_table const parse_table[] =
 #endif
   {"ctime", parse_ctime},
   {"daystart", parse_daystart},	/* GNU */
+  {"delete", parse_delete},	/* GNU, Mac OS, FreeBSD */
+  {"d", parse_d},		/* Mac OS X, FreeBSD, OpenBSD, but deprecated  in favour of -depth */
   {"depth", parse_depth},
   {"empty", parse_empty},	/* GNU */
   {"exec", parse_exec},
@@ -217,6 +221,7 @@ static struct parser_table const parse_table[] =
   {"print0", parse_print0},	/* GNU */
   {"printf", parse_printf},	/* GNU */
   {"prune", parse_prune},
+  {"quit",  parse_quit},	/* GNU */
   {"regex", parse_regex},	/* GNU */
   {"size", parse_size},
   {"true", parse_true},		/* GNU */
@@ -286,6 +291,9 @@ parse_and (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   our_pred = get_new_pred ();
   our_pred->pred_func = pred_and;
 #ifdef	DEBUG
@@ -324,6 +332,9 @@ parse_close (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   our_pred = get_new_pred ();
   our_pred->pred_func = pred_close;
 #ifdef	DEBUG
@@ -377,6 +388,9 @@ parse_comma (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+
   our_pred = get_new_pred ();
   our_pred->pred_func = pred_comma;
 #ifdef	DEBUG
@@ -399,6 +413,9 @@ parse_daystart (char **argv, int *arg_ptr)
 {
   struct tm *local;
 
+  (void) argv;
+  (void) arg_ptr;
+
   if (full_days == false)
     {
       cur_day_start += DAYSECS;
@@ -412,16 +429,47 @@ parse_daystart (char **argv, int *arg_ptr)
   return (true);
 }
 
+static boolean
+parse_delete (argv, arg_ptr)
+  char *argv[];
+  int *arg_ptr;
+{
+  struct predicate *our_pred;
+
+  our_pred = insert_primary (pred_delete);
+  our_pred->side_effects = true;
+  /* -delete implies -depth */
+  do_dir_first = false;
+  return (true);
+}
+
 static boolean
 parse_depth (char **argv, int *arg_ptr)
 {
+  (void) argv;
+  (void) arg_ptr;
+
   do_dir_first = false;
   return (true);
 }
  
+static boolean
+parse_d (char **argv, int *arg_ptr)
+{
+  (void) argv;
+  (void) arg_ptr;
+  error (0, 0,
+	 _("warning: the -d option is deprecated; please use -depth instead, because the latter is a POSIX-compliant feature."));
+  
+  return parse_depth(argv, arg_ptr);
+}
+ 
 static boolean
 parse_empty (char **argv, int *arg_ptr)
 {
+  (void) argv;
+  (void) arg_ptr;
+
   insert_primary (pred_empty);
   return (true);
 }
@@ -436,6 +484,9 @@ static boolean
 parse_false (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
+  
+  (void) argv;
+  (void) arg_ptr;
 
   our_pred = insert_primary (pred_false);
   our_pred->need_stat = false;
@@ -478,6 +529,9 @@ parse_fprintf (char **argv, int *arg_ptr)
 static boolean
 parse_follow (char **argv, int *arg_ptr)
 {
+  (void) argv;
+  (void) arg_ptr;
+
   dereference = true;
   xstat = stat;
   no_leaf_check = true;
@@ -565,6 +619,9 @@ parse_group (char **argv, int *arg_ptr)
 static boolean
 parse_help (char **argv, int *arg_ptr)
 {
+  (void) argv;
+  (void) arg_ptr;
+  
   printf (_("\
 Usage: %s [path...] [expression]\n"), program_name);
   puts (_("\
@@ -574,7 +631,7 @@ operators (decreasing precedence; -and is implicit where no others are given):\n
       ( EXPR ) ! EXPR -not EXPR EXPR1 -a EXPR2 EXPR1 -and EXPR2\n"));
   puts (_("\
       EXPR1 -o EXPR2 EXPR1 -or EXPR2 EXPR1 , EXPR2\n\
-options (always true): -daystart -depth -follow --help\n\
+options (always true): -daystart -delete -depth -follow --help\n\
       -maxdepth LEVELS -mindepth LEVELS -mount -noleaf --version -xdev\n\
 tests (N can be +N or -N or N): -amin N -anewer FILE -atime N -cmin N"));
   puts (_("\
@@ -701,6 +758,9 @@ parse_lname (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   if ((argv == NULL) || (argv[*arg_ptr] == NULL))
     return (false);
 
@@ -717,6 +777,9 @@ parse_ls (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) &argv;
+  (void) &arg_ptr;
+
   our_pred = insert_primary (pred_ls);
   our_pred->side_effects = true;
   our_pred->no_default_print = true;
@@ -789,6 +852,9 @@ parse_name (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   if ((argv == NULL) || (argv[*arg_ptr] == NULL))
     return (false);
   our_pred = insert_primary (pred_name);
@@ -803,6 +869,9 @@ parse_negate (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) &argv;
+  (void) &arg_ptr;
+
   our_pred = get_new_pred_chk_op ();
   our_pred->pred_func = pred_negate;
 #ifdef	DEBUG
@@ -820,6 +889,9 @@ parse_newer (char **argv, int *arg_ptr)
   struct predicate *our_pred;
   struct stat stat_newer;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   if ((argv == NULL) || (argv[*arg_ptr] == NULL))
     return (false);
   if ((*xstat) (argv[*arg_ptr], &stat_newer))
@@ -833,6 +905,9 @@ parse_newer (char **argv, int *arg_ptr)
 static boolean
 parse_noleaf (char **argv, int *arg_ptr)
 {
+  (void) &argv;
+  (void) &arg_ptr;
+  
   no_leaf_check = true;
   return true;
 }
@@ -858,6 +933,9 @@ parse_nogroup (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) &argv;
+  (void) &arg_ptr;
+  
   our_pred = insert_primary (pred_nogroup);
 #ifdef CACHE_IDS
   if (gid_unused == NULL)
@@ -890,6 +968,9 @@ static boolean
 parse_nouser (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
+  (void) argv;
+  (void) arg_ptr;
+  
 
   our_pred = insert_primary (pred_nouser);
 #ifdef CACHE_IDS
@@ -930,6 +1011,9 @@ parse_open (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   our_pred = get_new_pred_chk_op ();
   our_pred->pred_func = pred_open;
 #ifdef	DEBUG
@@ -946,6 +1030,9 @@ parse_or (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   our_pred = get_new_pred ();
   our_pred->pred_func = pred_or;
 #ifdef	DEBUG
@@ -1085,6 +1172,15 @@ parse_prune (char **argv, int *arg_ptr)
   return (true);
 }
 
+static boolean 
+parse_quit  (char **argv, int *arg_ptr)
+{
+  struct predicate *our_pred = insert_primary (pred_quit);
+  (*arg_ptr)++;
+  return true;
+}
+
+
 static boolean
 parse_regex (char **argv, int *arg_ptr)
 {
@@ -1108,19 +1204,17 @@ insert_regex (char **argv, int *arg_ptr, boolean ignore_case)
   re->allocated = 100;
   re->buffer = (unsigned char *) xmalloc (re->allocated);
   re->fastmap = NULL;
-
+  
   if (ignore_case)
     {
-      unsigned i;
-      
-      re->translate = xmalloc (256);
-      /* Map uppercase characters to corresponding lowercase ones.  */
-      for (i = 0; i < 256; i++)
-        re->translate[i] = ISUPPER (i) ? tolower (i) : i;
+      re_syntax_options |= RE_ICASE;
     }
   else
-    re->translate = NULL;
-
+    {
+      re_syntax_options &= ~RE_ICASE;
+    }
+  re->translate = NULL;
+  
   error_message = re_compile_pattern (argv[*arg_ptr], strlen (argv[*arg_ptr]),
 				      re);
   if (error_message)
@@ -1459,7 +1553,7 @@ insert_fprintf (FILE *fp, boolean (*func) (/* ??? */), char **argv, int *arg_ptr
 	  if (*scan2 == '.')
 	    for (scan2++; ISDIGIT (*scan2); scan2++)
 	      /* Do nothing. */ ;
-	  if (strchr ("abcdfFgGhHiklmnpPstuU", *scan2))
+	  if (strchr ("abcdDfFgGhHiklmnpPstuUyY", *scan2))
 	    {
 	      segmentp = make_segment (segmentp, format, scan2 - format,
 				       (int) *scan2);
@@ -1523,6 +1617,7 @@ make_segment (struct segment **segment, char *format, int len, int kind)
     case 'b':			/* size in 512-byte blocks */
     case 'c':			/* ctime in `ctime' format */
     case 'C':			/* ctime in user-specified strftime format */
+    case 'D':                   /* Filesystem device on which the file exits */
     case 'F':			/* filesystem type */
     case 'G':			/* GID number */
     case 'g':			/* group name */
@@ -1535,6 +1630,8 @@ make_segment (struct segment **segment, char *format, int len, int kind)
     case 'T':			/* mtime in user-specified strftime format */
     case 'U':			/* UID number */
     case 'u':			/* user name */
+    case 'y':			/* file type */
+    case 'Y':			/* symlink pointed file type */
       fprintf_stat_needed = true;
       /* FALLTHROUGH */
     case 'f':			/* basename of path */
@@ -1739,8 +1836,6 @@ insert_time (char **argv, int *arg_ptr, PFB pred)
 static boolean
 get_num (char *str, uintmax_t *num, enum comparison_type *comp_type)
 {
-  int len_num;			/* Length of field. */
-
   if (str == NULL)
     return (false);
   switch (str[0])
