@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 9 Temple Place - Suite 330, Boston, MA 02111-1307,
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
    USA.
 */
 
@@ -21,13 +21,6 @@
    inspired by John P. Rouillard <rouilj@cs.umb.edu>.  */
 
 #ifdef HAVE_CONFIG_H
-#include <gnulib/config.h>
-#undef VERSION
-#undef PACKAGE_VERSION
-#undef PACKAGE_TARNAME
-#undef PACKAGE_STRING
-#undef PACKAGE
-#undef PACKAGE_NAME
 #include <config.h>
 #endif
 
